//+------------------------------------------------------------------+
//|                                                         iDTW.mq4 |
//|                                                                * |
//|                                                                * |
//+------------------------------------------------------------------+
#property copyright "*"
#property link      "*"

#property indicator_chart_window
#property indicator_buffers 3
#property indicator_color1 Red
#property indicator_color2 Orange
#property indicator_color3 Yellow

#property indicator_width1 5
#property indicator_width2 3
#property indicator_width3 1

//--- input parameters
extern int       ExtParam1;
//--- buffers
double ExtMapBuffer1[];
double ExtMapBuffer2[];
double ExtMapBuffer3[];

double Matrix[1024][1024];
int Path[2048][2];


//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init(){
  
  ObjectsDeleteAll();
  
//---- indicators
   SetIndexStyle(0,DRAW_LINE);
   SetIndexBuffer(0,ExtMapBuffer1);
   SetIndexStyle(1,DRAW_LINE);
   SetIndexBuffer(1,ExtMapBuffer2);
   SetIndexStyle(2,DRAW_LINE);
   SetIndexBuffer(2,ExtMapBuffer3);
   
   SetIndexLabel(0,"Data1");
   SetIndexLabel(1,"Data2");
   SetIndexLabel(2,"Data2Transformed");
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
  {
//----
   ObjectsDeleteAll();
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start(){
   static bool solved=false;
   if(solved)return(0);
   solved=true;

   //===   ===

   int l1=100;
   int l2=50;

   double d1[];
   double d2[];
   
   ArrayResize(d1,l1);
   ArrayResize(d2,l2);

      for(int i=0;i<l1;i++){
         d1[i]=Open[0]+400*Point*(MathSin(6.28*i/l1*3+1)+0.25*MathSin(6.28*i/l1*30+1));
         ExtMapBuffer1[i]=d1[i];
      }
      
      for(i=0;i<l2;i++){
         d2[i]=Open[0]+400*Point*(MathSin(6.28*i/l2*2+2)+0.15*MathSin(6.28*i/l2*20+2));
         ExtMapBuffer2[i]=d2[i];         
      }   
      
   //=== DTW ===        
      
   int PathLen;      
      
   double dtw=DTWMatrix(d1,l1,d2,l2,Matrix,Path,PathLen);
   DTWPath(l1,l2,Matrix,Path,PathLen);
   
   // ===  ===
   
   //    2
      for(i=0;i<PathLen;i++){
         ExtMapBuffer3[Path[i][0]]=d2[Path[i][1]];
      }
   //          
   Draw(l1,l2,Matrix,Path,PathLen);     
         
   
   return(0);
}

//+------------------------------------------------------------------+

double DTWMatrix(double & aData1[],int aLength1,double & aData2[],int aLength2,double & aMatrix[][],int & Path[][],int & aPathLength){
	Matrix[0][0]=Distance(aData1[0],aData2[0]);
      for(int i=1;i<aLength1;i++){
         Matrix[i][0]=Matrix[i-1][0]+Distance(aData1[i],aData2[0]);
      }		    
      for(int j=1;j<aLength2;j++){
         Matrix[0][j]=Matrix[0][j-1]+Distance(aData1[0],aData2[j]);
      }            
      for(i=1;i<aLength1;i++){
         for(j=1;j<aLength2;j++){
            Matrix[i][j]=MathMin(Matrix[i-1][j],MathMin(Matrix[i][j-1],Matrix[i-1][j-1]))+Distance(aData1[i],aData2[j]);
         }
      }         				
   return (Matrix[aLength1-1][aLength2-1]);
}

void DTWPath(int aLength1,int aLength2,double & aMatrix[][],int & aPath[][],int & aPathLength){
   aPathLength=0;
   int i=aLength1;
   int j=aLength2;
      while(i>=1 && j>=1){
            if(i==1){
               j=j-1;
            }   
            else if(j==1){
               i=i-1;
            }
            else{
               if(Matrix[i-1][j]==MathMin(Matrix[i-1][j],MathMin(Matrix[i][j-1],Matrix[i-1][j-1])) && Matrix[i][j-1]==MathMin(Matrix[i-1][j],MathMin(Matrix[i][j-1],Matrix[i-1][j-1]))){
                  i=i-1; 
                  j=j-1;
               }
               else 
               if(Matrix[i][j-1]==MathMin(Matrix[i-1][j],MathMin(Matrix[i][j-1],Matrix[i-1][j-1]))){
                  j=j-1;
               }               
               else if(Matrix[i-1][j]==MathMin(Matrix[i-1][j],MathMin(Matrix[i][j-1],Matrix[i-1][j-1]))){
                  i=i-1;
               }   
               else{
                  i=i-1; 
                  j=j-1;
               }
            }
         aPath[aPathLength][0]=i;
         aPath[aPathLength][1]=j;
         aPathLength++;  
      }
   aPath[aPathLength][0]=0;
   aPath[aPathLength][1]=0;
   aPathLength++;  
}

double Distance(double Value1, double Value2) {
   return (MathAbs(Value1-Value2));
}

void Draw(int s1,int s2,double dst[][],int Path[][],int PathLen){
   double maxv=0;
   int i1,i2;
      for(i1=0;i1<s1;i1++){
         for(i2=0;i2<s2;i2++){
            maxv=MathMax(maxv,dst[i1][i2]);
         }
      } 
      for(i1=0;i1<s1;i1++){
         for(i2=0;i2<s2;i2++){
            double k=dst[i1][i2]/maxv;
            color col=GetColor(k,Red,Yellow);
            fObjLabel(WindowExpertName()+"_"+i1+"_"+i2,20+i1*2,20+i2*2,".",0,col,14,0,"Arial",false);
         }
      } 
      for(int i=0;i<PathLen;i++){
         i1=Path[i][0];
         i2=Path[i][1];
         fObjLabel(WindowExpertName()+"_"+i1+"_"+i2,20+i1*2,20+i2*2,".",0,Blue,14,0,"Arial",false); 
      }
}

void fObjLabel(
   string aObjectName,     // 1 
   int aX,                 // 2 
   int aY,                 // 3 
   string aText,           // 4 
   int aCorner=0,          // 5    0  1
                           //          2  3
   color aColor=Red,       // 6 
   int aFontSize=8,        // 7  
   int aWindowNumber=0,    // 8 
   string aFont="Arial",   // 9 
   bool aBack=false        // 10 
   ){     
   
   // fLabel("",10,10,"");
   // fLabel("",10,10,"",0,Red,8,0,"Arial",false);
    
      if(ObjectFind(aObjectName)!=aWindowNumber){
         ObjectCreate(aObjectName,OBJ_LABEL,aWindowNumber,0,0);
      }      
   ObjectSet(aObjectName,OBJPROP_XDISTANCE,aX);
   ObjectSet(aObjectName,OBJPROP_YDISTANCE,aY);   
   ObjectSetText(aObjectName,aText,aFontSize,aFont,aColor);
   ObjectSet(aObjectName,OBJPROP_BACK,aBack);
   ObjectSet(aObjectName,OBJPROP_CORNER,aCorner);   
}

color GetColor(double aK,int Col1,double Col2){
   int R1,G1,B1,R2,G2,B2;
   fGetRGB(R1,G1,B1,Col1);
   fGetRGB(R2,G2,B2,Col2);  
   return(fRGB(R1+aK*(R2-R1),G1+aK*(G2-G1),B1+aK*(B2-B1)));
}

void fGetRGB(int & aR,int & aG,int & aB,int aCol){
   aB=aCol/65536;
   aCol-=aB*65536;
   aG=aCol/256;
   aCol-=aG*256;
   aR=aCol;
}

color fRGB(int aR,int aG, int aB){
   return(aR+256*aG+65536*aB);
}

